package gcewing.prospecting;

import java.util.Map;
import java.util.WeakHashMap;
import net.minecraft.src.*;

public class ItemSlimophone extends Item {

	static int maxBatteryCharge = 100000;
	static double maximumRange = 160.0; // blocks
	static String pingSound = "mob.slime"; //"gcewing.prospecting.resources.slimefilt";
	static double omnidirectionalGain = 0.2;
	static int numBars = 6;
	static double barDecayFactor = 0.9;
	static int cooldownTime = 30; // ticks
	
	public static double barIntensities[] = new double[numBars];
	
	static boolean scanPerformed;
	static Map<Entity, SlimeData> slimeData = new WeakHashMap<Entity, SlimeData>();

	public ItemSlimophone(int id) {
		super(id);
		setMaxStackSize(1);
		setMaxDamage(maxBatteryCharge);
		setIconIndex(ModLoader.addOverride("/gui/items.png", "/gcewing/prospecting/resources/slimophone.png"));
	}
	
	public static void onTick() {
		decayBars();
		scanPerformed = false;
	}

	public void onUpdate(ItemStack stack, World world, Entity entity, int slotNum, boolean inHand) {
		//System.out.printf("ItemSlimophone.onUpdate: %d entities\n",
		//	world.loadedEntityList.size());
		if (hasPower(stack)) {
			if (!world.isRemote) {
				//System.out.printf("ItemSlimophone: Updating battery\n");
				updateBattery(stack);
			}
			if (!GregsProspecting.isServer) {
				//System.out.printf("ItemSlimophone: Checking whether scan required\n");
				if (!scanPerformed) {
					//System.out.printf("ItemSlimophone: Performing scan\n");
					performScan(stack, world, entity, slotNum, inHand);
					scanPerformed = true;
				}
			}
		}
	}
	
	void performScan(ItemStack stack, World world, Entity entity, int slotNum, boolean inHand) {
		double maxSignal = 0.0;
		if (entity instanceof EntityPlayer) {
			EntityPlayer player = (EntityPlayer) entity;
			for (Object obj : world.loadedEntityList) {
				//System.out.printf("ItemSlimophone.performScan: found %s\n", obj.getClass().getName());
				if (obj instanceof EntitySlime) {
					EntitySlime slime = (EntitySlime) obj;
					//System.out.printf("   Found slime at (%.2f,%.2f,%.2f) isCollidedVertically = %s\n",
					//	slime.posX, slime.posY, slime.posZ, slime.isCollidedVertically);
					SlimeData data = getSlimeData(slime);
					if (data.cooldownTimer > 0) {
						//System.out.printf("   Still cooling down\n");
						data.cooldownTimer -= 1;
					}
					else {
						if (slime.posY != data.posY) {
							data.posY = slime.posY;
							data.cooldownTimer = cooldownTime;
							Vec3D slimePos = Vec3D.createVector(slime.posX, slime.posY, slime.posZ);
							Vec3D playerPos = Vec3D.createVector(player.posX, player.posY, player.posZ);
							double distFromPlayer = playerPos.distanceTo(slimePos);
							//System.out.printf("   Distance from player = %s\n", distFromPlayer);
							if (distFromPlayer <= maximumRange) {
								Vec3D player2slime = slimePos.addVector(-playerPos.xCoord, -playerPos.yCoord, -playerPos.zCoord);
								Vec3D look = player.getLookVec();
								//System.out.printf("   To slime = %s\n", player2slime);
								//System.out.printf("   Look = %s\n", look);
								double gain = Math.max(player2slime.normalize().dotProduct(look), omnidirectionalGain);
								double signal = gain * (1.0 - distFromPlayer / maximumRange);
								//System.out.printf("   Signal = %s, gain = %s\n", signal, gain);
								world.playSoundAtEntity(player, pingSound, (float) signal, 1.0F);
								maxSignal = Math.max(maxSignal, signal);
							}
						}
					}
				}
			}
		}
		if (maxSignal > 0.0)
			intensifyBars(maxSignal);
	}
	
	void dumpBarIntensities() {
		System.out.printf("ItemSlimophone: Bar intensities =");
		for (int i = 0; i < numBars; i++)
			System.out.printf(" %.2f", barIntensities[i]);
		System.out.printf("\n");
	}
	
	void intensifyBars(double signal) {
		int n = (int) Math.round(signal * numBars);
		//System.out.printf("ItemSlimophone: Intensifying %d bars\n", n);
		for (int i = 0; i < n; i++)
			barIntensities[i] = 1.0;
	}
	
	static void decayBars() {
		for (int i = 0; i < numBars; i++)
			barIntensities[i] *= barDecayFactor;
	}
	
	boolean hasPower(ItemStack stack) {
		return stack.getItemDamage() < stack.getMaxDamage();
	}
	
	void updateBattery(ItemStack stack) {
		stack.setItemDamage(stack.getItemDamage() + 1);
	}
	
	SlimeData getSlimeData(Entity slime) {
		SlimeData data = slimeData.get(slime);
		if (data == null) {
			data = new SlimeData();
			slimeData.put(slime, data);
		}
		return data;
	}
	
}

class SlimeData {

	public double posY;
	public int cooldownTimer;
	
}

