//------------------------------------------------------
//
//   Greg's Prospecting Mod - Common
//
//------------------------------------------------------

package net.minecraft.src;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;
import java.util.Hashtable;
import gcewing.prospecting.*;

public abstract class GregsProspecting extends BaseMod {

	public static String modName = "GregsProspecting";
	public static String channelName = "gce.prospecting";
	public static String version = "1.3";

	public static ItemMiningRadar miningRadar, colorMiningRadar;
	public static Item battery, mandrel, batteryCasing, carbonRod,
		electrolytePaste, bowlAppleJuice, carbonFilament, cathodeRayTube, magnetron,
		fibreglassBoard, microwaveHorn, brokenMiningRadar, slimophone, parabolicMicrophone,
		piezoTransducer, quartzWafer, triode, ledBarGraph, redPhosphor, greenPhosphor, bluePhosphor,
		colorCRT, colorMiningRadarCircuit;
	
	static String configName = modName + ".cfg";
	static File cfgfile;
	static OrderedProperties config;
	static Map <Integer, String> idToName = new Hashtable <Integer, String>();
	
	public static boolean isServer;
	
	GregsProspecting(boolean server) {
		isServer = server;
	}

	public void load(File mcdir) {
		loadConfig(mcdir);
		registerBlocks();
		registerItems();
		addRecipes();
		registerChannels();
		saveConfig();
	}
	
	void loadConfig(File mcdir) {
		File cfgdir = new File(mcdir, "/config/");
		cfgfile = new File(cfgdir, configName);
		config = new OrderedProperties();
		try {
			config.load(new FileInputStream(cfgfile));
		}
		catch (FileNotFoundException e) {
			System.out.printf("GregsProspecting: No existing config file\n");
		}
		catch (IOException e) {
			System.out.printf("GregsProspecting: Failed to read %s\n%s\n",
				cfgfile.getPath(), e);
		}
		config.extended = false;
	}
	
	void saveConfig() {
		try {
			if (config.extended) {
			//if (cfgfile.exists() || cfgfile.createNewFile()) {
				System.out.printf("GregsProspecting: Writing config file\n");
				config.store(new FileOutputStream(cfgfile),
					modName + " Configuration File");
				//System.out.printf("GregsProspecting: Written config file\n");
			}
		}
		catch (IOException e) {
			System.out.printf("GregsProspecting: Failed to %s\n%s\n",
				cfgfile.getPath(), e);
		}
	}

	public String getVersion() {
		return version;
	}
	
	void registerChannels() {
		ModLoader.registerPacketChannel(this, channelName);
	}
	
	void registerBlocks() {	
	}
	
	void registerItems() {
		miningRadar = new ItemMiningRadar(getItemID(3000, "miningRadar"), "mining_radar.png");
		addItem("Mining Radar", miningRadar);
		battery = addItem("Battery", newItemSI(3001, "battery", 6, "battery.png"));
		mandrel = addItem("Wooden Mandrel", newItemI(3002, "mandrel", "wooden_mandrel.png"));
		mandrel.setContainerItem(mandrel);
		batteryCasing = addItem("Battery Casing", newItemI(3003, "batteryCasing", "battery_casing.png"));
		carbonRod = addItem("Carbon Rod", newItemI(3004, "carbonRod", "carbon_rod.png"));
		electrolytePaste = addItem("Electrolyte Paste", newItemI(3005, "electrolytePaste", "electrolyte_paste.png"));
		bowlAppleJuice = addItem("Apple Juice", newItemI(3006, "bowlAppleJuice", "bowl_of_apple_juice.png"));
		bowlAppleJuice.setContainerItem(Item.bowlEmpty);
		carbonFilament = addItem("Carbon Filament", newItemI(3007, "carbonFilament", "carbon_filament.png"));
		cathodeRayTube = addItem("Cathode Ray Tube", newItemI(3008, "cathodeRayTube", "cathode_ray_tube.png"));
		magnetron = addItem("Magnetron", newItemI(3009, "magnetron", "magnetron.png"));
		fibreglassBoard = addItem("Fibreglass Board", newItemI(3010, "fibreglassBoard", "fibreglass_board.png"));
		microwaveHorn = addItem("Microwave Horn", newItemI(3011, "microwaveHorn", "microwave_horn.png"));
		brokenMiningRadar = addItem("Broken Mining Radar", newItemI(3012, "brokenMiningRadar", "broken_mining_radar.png"));
		slimophone = addItem("Slimophone", new ItemSlimophone(getItemID(3013, "slimophone")));
		parabolicMicrophone = addItem("Parabolic Microphone", newItemI(3014, "parabolicMicrophone", "parabolic_microphone.png"));
		piezoTransducer = addItem("Piezo Transducer", newItemI(3015, "piezoTransducer", "piezo_transducer.png"));
		quartzWafer = addItem("Quartz Wafer", newItemI(3016, "quartzWafer", "quartz_wafer.png"));
		triode = addItem("Triode", newItemI(3017, "triode", "triode.png"));
		ledBarGraph = addItem("LED Bar Graph", newItemI(3018, "ledBarGraph", "led_bar_graph.png"));
		colorMiningRadar = new ItemMiningRadar(getItemID(3019, "colorMiningRadar"), "color_mining_radar.png");
		addItem("Colour Mining Radar", colorMiningRadar);
		redPhosphor = addItem("Red Phosphor", newItemI(3020, "redPhosphor", "red_phosphor.png"));
		greenPhosphor = addItem("Green Phosphor", newItemI(3021, "greenPhosphor", "green_phosphor.png"));
		bluePhosphor = addItem("Blue Phosphor", newItemI(3022, "bluePhosphor", "blue_phosphor.png"));
		colorCRT = addItem("Colour CRT", newItemI(3023, "colorCRT", "color_crt.png"));
		colorMiningRadarCircuit = addItem("Colour Mining Radar Circuit", newItemI(3024, "colorMiningRadarCircuit", "color_mr_circuit.png"));
	}

	void addRecipes() {
		addShapelessRecipe(miningRadar, 1, new ItemStack(miningRadar, 1, -1), battery);
		addRecipe(mandrel, 1, "W", "W", "W", 'W', Block.planks);
		addShapelessRecipe(batteryCasing, 6, Item.ingotIron, mandrel);
		addRecipe(carbonRod, 6, "C", "C", "C", 'C', Item.coal);
		addShapelessRecipe(bowlAppleJuice, 1, Item.appleRed, Item.bowlEmpty);
		addShapelessRecipe(electrolytePaste, 6, bowlAppleJuice, new ItemStack(Item.dyePowder, 1, 15));
		addShapelessRecipe(battery, 1, batteryCasing, electrolytePaste, carbonRod);
		addSmeltingRecipe(carbonFilament, 1, Item.silk);
		addRecipe(cathodeRayTube, 1, "|gG", "|gf", "|gG",
			'|', Block.thinGlass, 'g', Item.lightStoneDust, 'G', Block.glass, 'f', carbonFilament);
		addRecipe(magnetron, 1, "rGr", "IGI", "rfr",
			'r', Item.redstone, 'G', Block.glass, 'I', Item.ingotIron, 'f', carbonFilament);
		addShapelessRecipe(fibreglassBoard, 1, Block.glass, Item.slimeBall);
		addRecipe(microwaveHorn, 1, "  I", "II ", "  I", 'I', Item.ingotIron);
		addRecipe(miningRadar, 1, "CMH", "LFB", "grg",
			'C', cathodeRayTube, 'M', magnetron, 'H', microwaveHorn, 'L', Block.lever, 'F', fibreglassBoard,
			'B', battery, 'g', Item.goldNugget, 'r', Item.redstone);
		addRecipe(slimophone, 1, "ltp", "grb", 'l', ledBarGraph, 't', triode, 'p', parabolicMicrophone,
			'g', Item.goldNugget, 'r', Item.redstone, 'b', battery);
		addRecipe(ledBarGraph, 1, "rgB", "rgB", "rgB",
			'r', Item.redstone, 'g', Item.lightStoneDust, 'B', new ItemStack(Block.cloth, 1, 15));
		addShapelessRecipe(slimophone, 1, new ItemStack(slimophone, 1, -1), battery);
		addRecipe(parabolicMicrophone, 1, " W", "Wp", " W", 'W', Block.planks, 'p', piezoTransducer);
		addRecipe(piezoTransducer, 1, "IqI", 'I', Item.ingotIron, 'q', quartzWafer);
		addSmeltingRecipe(quartzWafer, 1, Block.sandStone);
		addRecipe(triode, 1, " I ", "GrG", " f ", 
			'I', Item.ingotIron, 'G', Block.glass, 'r', Item.redstone, 'f', carbonFilament);
		addShapelessRecipe(redPhosphor, 1, Item.blazePowder, new ItemStack(Item.dyePowder, 1, 1));
		addShapelessRecipe(greenPhosphor, 1, Item.blazePowder, new ItemStack(Item.dyePowder, 1, 2));
		addShapelessRecipe(bluePhosphor, 1, Item.blazePowder, new ItemStack(Item.dyePowder, 1, 4));
		addRecipe(colorCRT, 1, "|rf", "|gf", "|bf",
			'|', Block.thinGlass, 'r', redPhosphor, 'g', greenPhosphor, 'b', bluePhosphor, 'f', carbonFilament);
		addRecipe(colorMiningRadarCircuit, 1, "rrr", " b ", " I ",
			'r', Item.redstone, 'b', fibreglassBoard, 'I', Item.ingotIron);
		addShapelessRecipe(colorMiningRadar, 1, miningRadar, colorCRT, colorMiningRadarCircuit);
		addRecipe(colorMiningRadar, 1, "CMH", "LFB", "grg",
			'C', colorCRT, 'M', magnetron, 'H', microwaveHorn, 'L', Block.lever, 'F', colorMiningRadarCircuit,
			'B', battery, 'g', Item.goldNugget, 'r', Item.redstone);
		addShapelessRecipe(colorMiningRadar, 1, new ItemStack(colorMiningRadar, 1, -1), battery);
	}
	
	void addRecipe(Item product, int qty, Object... params) {
		ModLoader.addRecipe(new ItemStack(product, qty), params);
	}

	void addShapelessRecipe(Item product, int qty, Object... params) {
		ModLoader.addShapelessRecipe(new ItemStack(product, qty), params);
	}

	void addSmeltingRecipe(Item product, int qty, Item input) {
		ModLoader.addSmelting(input.shiftedIndex, new ItemStack(product, qty));
	}
	
	void addSmeltingRecipe(Item product, int qty, Block input) {
		ModLoader.addSmelting(input.blockID, new ItemStack(product, qty));
	}
	
	static int getItemID(int defaultID, String name) {
		int id;
		if (config.containsKey(name)) {
			String value = (String) config.get(name);
			id = Integer.parseInt(value);
		}
		else {
			config.put(name, Integer.toString(defaultID));
			id = defaultID;
		}
		idToName.put(Integer.valueOf(256 + id), name);
		return id;
	}

	Item newItem(int id, String name) {
		return new Item(getItemID(id, name));
	}
	
	Item newItemI(int id, String name, String icon) {
		return new Item(getItemID(id, name)).setIconIndex(iconOverride(icon));
	}
	
	Item newItemSI(int id, String name, int maxStack, String icon) {
		return new Item(getItemID(id, name)).setMaxStackSize(maxStack).setIconIndex(iconOverride(icon));
	}
	
	int iconOverride(String name) {
		return ModLoader.addOverride("/gui/items.png", "/gcewing/prospecting/resources/" + name);
	}
	
	Item addItem(String name, Item item) {
		item.setItemName("gcewing." + idToName.get(item.shiftedIndex));
		//System.out.printf("Name of item %d set to %s\n", item.shiftedIndex - 256, item.getItemName());
		ModLoader.addName(item, name);
		return item;
	}
	
	void performMiningRadarAction(EntityPlayer player, MiningRadarAction action) {
		if (player != null) {
			ItemStack stack = player.getCurrentEquippedItem();
			if (stack != null) {
				Item item = stack.getItem();
				if (item != null && item instanceof ItemMiningRadar)
					((ItemMiningRadar)item).performAction(stack, action);
				}
		}
	}
	
	public static boolean entityIsJumping(EntityLiving entity) {
		// Defined here to get around protected access of EntityLiving.isJumping
		return entity.isJumping;
	}

}
